from sysactions import get_model, get_posot

from application.model.order import Order


class VerifyAndRemoveBenefitsUnitOfWork(object):
    def __init__(self, pos_id, order):
        # type: (int, Order) -> None

        self.pos_id = pos_id
        self.model = get_model(self.pos_id)
        self.pos_ot = get_posot(self.model)
        self.order = order
