from helper import ExtendedEnum
from messagebus import TokenCreator, TokenPriority

MESSAGE_GROUP = "112"


class ListenedTokens(ExtendedEnum):
    TK_NDISCOUNT_ADD_BENEFIT = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "1")
    TK_NDISCOUNT_REMOVE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "2")
    TK_NDISCOUNT_GET_BENEFITS = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "7")
    TK_NDISCOUNT_ON_ITEM_SALE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "8")
    TK_NDISCOUNT_ON_ORDER_TOTAL = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "9")
    TK_NDISCOUNT_ON_ORDER_REOPEN = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "10")
    TK_NDISCOUNT_ON_TABLE_REOPEN = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "12")
    TK_NDISCOUNT_ON_TABLE_OPEN = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "13")
