-- Schema: production
-- Brief: used by the Production component to persist the orders current state
-- Created by: rrosauro
-- Table schema_version: must be defined for database update through the Persistence Component
CREATE TABLE schema_version AS
SELECT "$Author: rrosauro $" AS Author,
       "$Date: 2010-02-03 10:13:59 -0200 (Wed, 03 Feb 2010) $" AS LastModifiedAt,
       "$Revision: 4247a7616784f3f0697186df3251dc348bb25292$" AS Revision;
       
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

CREATE TABLE ProductionOrders (
	OrderId				INTEGER PRIMARY KEY,
	Object				BLOB NOT NULL,
	CreatedAt			TIMESTAMP NOT NULL,
	Purged				BOOLEAN NOT NULL DEFAULT 0
);

-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
