from dailygoals_services.model import SaleChallenge
from dailygoals_services.model.repository import DailyGoalsRepository


class UpdateSalesChallengeInteractor:
    def __init__(self, daily_goals_repository):
        # type: (DailyGoalsRepository) -> None

        self.daily_goals_repository = daily_goals_repository

    def update_sales_challenge(self, challenge):
        # type: (SaleChallenge) -> int

        return self.daily_goals_repository.update_sales_challenge(challenge)
