class InvalidRequestException(Exception):
    pass


class CustomerExistsException(Exception):
    pass


class CustomerNonExistsException(Exception):
    pass


class InsertCustomerException(Exception):
    pass


class UpdateCustomerException(Exception):
    pass
