# -*- coding: utf-8 -*-

from models import NfeCardBrand


class PosTenderBrandToNfeCardBrand(object):
    @staticmethod
    def convert(pos_tender_brand):
        # type: (unicode) -> str

        tender_brand_map = {

        }

        nfe_card_brand = tender_brand_map.get(pos_tender_brand, NfeCardBrand.outros)
        return str(nfe_card_brand)
