class DeviceNotConnected(Exception):
    pass


class UnknownDevice(Exception):
    pass


class UnmappedDeviceServiceThread(Exception):
    pass


class FailedWritingToDevice(Exception):
    pass


class PrinterDoorOpened(Exception):
    pass


class PrinterNotConnected(Exception):
    pass
